# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.mixins - implements common mixin classes."""

from functools import total_ordering

from .exceptions import ExtractError
from .utils import identity


class ContextManager(object):
    """Mixin class that support ``with`` statement for monad."""
    # pylint: disable = too-few-public-methods
    def __enter__(self):
        if not self:
            raise ExtractError(self)
        return self.bind(identity)

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False

    bind = NotImplemented


@total_ordering
class Ord(object):
    """Mixin class that implements rich comparison ordering methods."""
    # pylint: disable = too-few-public-methods
    def __eq__(self, other):
        if self is other:
            return True
        elif not isinstance(other, type(self)):
            return NotImplemented
        else:
            return self.value == other.value

    def __lt__(self, other):
        if self is other:
            return False
        elif isinstance(other, type(self)):
            return self.value < other.value
        else:
            fmt = "unorderable types: {} and {}'".format
            raise TypeError(fmt(type(self), type(other)))

    value = NotImplemented



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.functor - The Functor Class."""


class Functor(object):
    """The Functor Class.

    Defines function ``fmap``, and should satisfy these laws::

        fmap id  ==  id
        fmap (f . g)  ==  fmap f . fmap g
    """
    # pylint: disable = too-few-public-methods
    def __init__(self, value):
        self.value = value

    def fmap(self, function):
        """The fmap operation."""
        raise NotImplementedError



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import os
import sys


sys.path.insert(0, os.path.abspath(os.path.dirname(os.path.dirname(__file__))))



# -*- coding: utf-8 -*-
#
# Monad documentation build configuration file.
import os
import sys

PROJECT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '../'))
sys.path.insert(0, PROJECT_DIR)
import smonad

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.viewcode',
]

source_suffix = '.rst'

master_doc = 'index'

project = u'smonad'
copyright = u'2012-2014, Philip Xu'

version = '%d.%d' % smonad.__version__
release = smonad.VERSION

exclude_patterns = ['_build']

pygments_style = 'sphinx'

html_theme = 'agogo'
# use RTD new theme
RTD_NEW_THEME = True

htmlhelp_basename = 'Monaddoc'

latex_documents = [
    ('index', 'SMonad.tex', u'SMonad Documentation',
     u'Philip Xu', 'manual'),
]

man_pages = [
    ('index', 'smonad', u'SMonad Documentation',
     [u'Philip Xu'], 1)
]

texinfo_documents = [
    ('index', 'SMonad', u'smonad Documentation',
     u'Philip Xu', 'SMonad', smonad.__doc__,
     'Miscellaneous'),
]



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.types import Functor
from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Failure, Success
from smonad.types import List

testee = [
    Identity,
    Maybe,
    Failure,
    Success,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'functor' in metafunc.funcargnames:
        metafunc.parametrize('functor', testee, ids=ids)


def test_functor_is_abstract():
    with pytest.raises(NotImplementedError):
        Functor(1).fmap(lambda a: a)


def test_fmap_functor_laws(functor):
    identity = lambda a: a
    f = lambda a: a + 1
    g = lambda a: a * 2
    f_g = lambda n: f(g(n))

    for n in test_range:
        ft = functor(n)
        # fmap id == id
        assert ft.fmap(identity) == identity(ft)
        # fmap (f . g) == fmap f . fmap g
        assert ft.fmap(f_g) == ft.fmap(g).fmap(f)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.list - The List Monad."""

from collections import Sequence
from itertools import chain

from . import LazySequence, MonadPlus
from ..mixins import Ord


class List(MonadPlus, Ord, Sequence):
    """The List Monad.

    Representing nondeterministic computation.

    >>> List(42)
    List(42)
    >>> List(1, 2, 3)
    List(1, 2, 3)
    >>> List([])
    List([])
    >>> List.from_iterable(range(3))
    List(0, 1, 2)
    >>> List.from_iterable(n for n in (1, 2, 3) if n % 2 == 0)
    List(2)
    >>> List(List(2))
    List(List(2))

    Lists are lazy

    >>> from itertools import count
    >>> m = List.from_iterable(count())
    >>> m[:5]
    List(0, 1, 2, 3, 4)
    >>> m[520:524]
    List(520, 521, 522, 523)
    >>> list(m[1000:1002])
    [1000, 1001]

    Bind operation with ``>>``

    >>> spawn = lambda cell: List(cell, cell)
    >>> spawn('c')
    List('c', 'c')
    >>> spawn('c') >> spawn
    List('c', 'c', 'c', 'c')
    >>> grow = lambda cell: List(cell + '~')
    >>> grow('o')
    List('o~')
    >>> grow('o') >> grow >> grow >> grow
    List('o~~~~')
    >>> generation = lambda cell: grow(cell) + spawn(cell)
    >>> first = List('o')
    >>> first
    List('o')
    >>> first >> generation
    List('o~', 'o', 'o')
    >>> first >> generation >> generation
    List('o~~', 'o~', 'o~', 'o~', 'o', 'o', 'o~', 'o', 'o')
    """
    # pylint: disable = too-many-ancestors
    def __init__(self, *items):
        super(List, self).__init__(LazySequence(items))

    def __getitem__(self, index):
        items = self.value.__getitem__(index)
        if isinstance(index, slice):
            return self.from_iterable(items)
        return items

    def __len__(self):
        return len(self.value)

    def __nonzero__(self):
        return len(self.value)

    def __repr__(self):
        """Customized Show."""
        return 'List({})'.format(', '.join(repr(x) for x in self))

    @classmethod
    def from_iterable(cls, iterator):
        """Creates ``List`` from iterable."""
        instance = cls.unit()
        instance.value = LazySequence(iterator)
        return instance

    def fmap(self, function):
        """fmap of List Monad."""
        return self.from_iterable(function(i) for i in self)

    def join(self):
        """join of List Monad."""
        return self.from_iterable(chain.from_iterable(self))

    def plus(self, monad):
        """plus operation, concatenates two ``List``."""
        if not isinstance(monad, type(self)):
            return NotImplemented
        return self.from_iterable(chain(self, monad))


List.zero = List()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.decorators import monadic
from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Try
from smonad.types import List

testee = [
    Identity,
    Maybe,
    Try,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'monad' in metafunc.funcargnames:
        metafunc.parametrize('monad', testee, ids=ids)


def test_monadic_kleisli_composition_operator_type_check():
    with pytest.raises(TypeError):
        monadic(lambda a: a) >> 1


def test_left_to_right_kleisli_composition(monad):
    unit = monad.unit
    add_1 = monadic(lambda n: unit(n + 1))
    double = monadic(lambda n: unit(n * 2))
    action = add_1 >> double
    for n in test_range:
        m = unit(n)
        assert m >> action == m >> add_1 >> double


def test_right_to_left_kleisli_composition(monad):
    unit = monad.unit
    add_1 = monadic(lambda n: unit(n + 1))
    double = monadic(lambda n: unit(n * 2))
    action = double << add_1
    for n in test_range:
        m = unit(n)
        assert m >> action == m >> add_1 >> double


def test_chain_right_to_left_kleisli_composition(monad):
    unit = monad.unit
    add_1 = monadic(lambda n: unit(n + 1))
    double = monadic(lambda n: unit(n * 2))
    for n in test_range:
        m = unit(n)
        assert add_1 << add_1 << m == m >> add_1 >> add_1
        assert double << double << m == m >> double >> double
        assert add_1 << double << m == m >> double >> add_1
        assert double << add_1 << m == m >> add_1 >> double



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.types import MonadPlus
from smonad.types import Maybe
from smonad.types import List

testee = [
    Maybe,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'monadplus' in metafunc.funcargnames:
        metafunc.parametrize('monadplus', testee, ids=ids)


def test_monadplus_is_abstract():
    with pytest.raises(NotImplementedError):
        MonadPlus(1).plus(1)


# Note:
# As stated on haskell wiki: http://www.haskell.org/haskellwiki/MonadPlus
# The precise set of rules that MonadPlus should obey is not agreed upon.
# More discussion here:
# http://www.haskell.org/haskellwiki/MonadPlus_reform_proposal
# This is my own interpretation, especially the implementation of Maybe Monad
# has unbiased plus, which satisfy Monoid and *Left Distribution*.

def test_zero_implemented(monadplus):
    zero = monadplus.zero
    assert zero is monadplus.zero


def test_plus_implemented(monadplus):
    zero = monadplus.zero
    assert zero.plus(zero) == zero


def test_plus_operator(monadplus):
    unit = monadplus.unit
    for n in test_range:
        m = unit(n)
        assert m + unit(n) == m.plus(unit(n))


def test_plus_zero_form_a_monoid(monadplus):
    unit = monadplus.unit
    zero = monadplus.zero
    for n in test_range:
        m = unit(n)
        # zero is a neutral element
        # zero + m = m
        assert zero + m == m
        assert zero.plus(m) == m
        # m + zero = m
        assert m + zero == m
        assert m.plus(zero) == m


def test_left_zero(monadplus):
    unit = monadplus.unit
    zero = monadplus.zero
    f = lambda n: unit(n + 1)

    # zero >>= f = zero
    assert zero >> f == zero

    # v >> zero = zero
    # Not testing this since the >> we implemented is actually >>= in
    # haskell, if we really want to do it, here is how:
    # for n in test_range:
    #     bind_and_discard = lambda m, other_m: other_m
    #     v = unit(n)
    #     assert bind_and_discard(v, zero) == zero


def test_left_distribution(monadplus):
    unit = monadplus.unit
    k = lambda n: unit(n + 1)
    # (m `mplus` n) >>= k  =  (m >>= k) `mplus` (n >>= k)
    for a in test_range:
        m = unit(a)
        n = unit(a * 2)
        assert (m + n) >> k == (m >> k) + (n >> k)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
from itertools import count

import pytest

from smonad.types import LazySequence


def test_lazy_sequence_initialization():
    # no need to use assert here, if LazySequence is not lazy, execution will
    # never stop in for statement.
    infinite = LazySequence(count())
    for n in infinite:
        if n == 42:
            break


def test_lazy_sequence_iterator():
    numbers = LazySequence(n for n in range(10))

    iter1 = iter(numbers)
    iter2 = iter(numbers)

    for index, (i1, i2) in enumerate(zip(iter1, iter2)):
        assert index == i1 == i2


def test_lazy_sequence_eq():
    seq = LazySequence([1, 2])

    assert seq == seq
    assert seq == LazySequence([1, 2])

    not_seq = object()
    assert (seq == not_seq) is False


def test_lazy_sequence_ording():
    seq = LazySequence([1, 2])
    assert (seq < seq) is False

    class NotSeq:
        pass

    with pytest.raises(TypeError):
        seq < NotSeq()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
from smonad.types import Function
from smonad.decorators import function


def test_callable():
    to_int = Function(int)
    assert to_int('42') == 42
    to_int = Function(lambda n: int(n))
    assert to_int('42') == 42

    class ToInt(object):
        def __call__(self, n):
            return int(n)

    to_int = Function(ToInt())
    assert to_int('42') == 42


def test_composition_operator():
    to_int = Function(int)
    add_1 = Function(lambda n: n + 1)
    to_s = Function(str)

    inc = add_1 * to_int
    assert inc('42') == add_1(to_int('42'))

    inc_s = to_s * add_1 * to_int
    assert inc_s('42') == to_s(add_1(to_int('42')))


def test_pipe_operator():
    to_int = Function(int)
    add_1 = Function(lambda n: n + 1)
    to_s = Function(str)

    inc = to_int | add_1
    assert inc('42') == add_1(to_int('42'))

    inc_s = to_int | add_1 | to_s
    assert inc_s('42') == to_s(add_1(to_int('42')))


def test_composition_auto_promote():
    add_1 = Function(lambda n: n + 1)

    inc = add_1 * int
    assert inc('42') == 43
    inc = int | add_1
    assert inc('42') == 43

    inc_s = str * add_1
    assert inc_s(42) == '43'
    inc_s = add_1 | str
    assert inc_s(42) == '43'


def test_composition_pipe_equivalent():
    add_1 = Function(lambda n: n + 1)
    assert (add_1 * int)('42') == (int | add_1)('42')


def test_compose():
    identity = lambda a: a
    # Left identity
    f = Function(int)
    g = identity * f

    assert f('42') == g('42')

    for n in range(42):  # this number is not significant here.
        assert f(n) == g(n)

    # Right identity
    f = Function(str)
    g = f * identity

    assert f(42) == g(42)

    for n in range(42):  # this number is not significant here.
        assert f(n) == g(n)

    # Associative

    # NOTE: these functions are specifically chosen, so that when composed in
    # different orders, yield different outputs.
    f = Function(hash)
    g = Function(str)
    h = Function(type)

    f1 = f * (g * h)
    f2 = (f * g) * h

    ns = [42, 0.5, '42', sum, f, lambda: 0]
    for n in ns:
        # make sure these functions yield different outputs when composed in
        # different order,
        assert f1(n) != (f * (h * g))(n)
        assert f2(n) != (f * (h * g))(n)
        assert f1(n) != (g * (f * h))(n)
        assert f2(n) != (g * (f * h))(n)
        assert f1(n) != (g * (h * f))(n)
        assert f2(n) != (g * (h * f))(n)
        assert f1(n) != (h * (g * f))(n)
        assert f2(n) != (h * (g * f))(n)
        assert f1(n) != (h * (f * g))(n)
        assert f2(n) != (h * (f * g))(n)
        # but compose should obey associative law.
        assert f1(n) == f2(n)


def test_decorator():
    @function
    def to_int(n):
        return int(n)

    assert to_int('42') == 42



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.utils import compose, identity, suppress


def test_compose():
    # Left identity
    f = int
    g = compose(identity, f)

    assert f('42') == g('42')

    for n in range(42):  # this number is not significant here.
        assert f(n) == g(n)

    # Right identity
    f = str
    g = compose(f, identity)

    assert f(42) == g(42)

    for n in range(42):  # this number is not significant here.
        assert f(n) == g(n)

    # Associative

    # NOTE: these functions are specifically chosen, so that when composed in
    # different orders, yield different outputs.
    f = hash
    g = str
    h = type

    f1 = compose(f, compose(g, h))
    f2 = compose(compose(f, g), h)

    ns = [42, 0.5, '42', sum, f, lambda: 0]
    for n in ns:
        # make sure these functions yield different outputs when composed in
        # different order,
        assert f1(n) != compose(f, compose(h, g))(n)
        assert f2(n) != compose(f, compose(h, g))(n)
        assert f1(n) != compose(g, compose(f, h))(n)
        assert f2(n) != compose(g, compose(f, h))(n)
        assert f1(n) != compose(g, compose(h, f))(n)
        assert f2(n) != compose(g, compose(h, f))(n)
        assert f1(n) != compose(h, compose(g, f))(n)
        assert f2(n) != compose(h, compose(g, f))(n)
        assert f1(n) != compose(h, compose(f, g))(n)
        assert f2(n) != compose(h, compose(f, g))(n)
        # but compose should obey associative law.
        assert f1(n) == f2(n)


def test_identity():
    for n in [0, 1, 'a', [], {}, None, True, False, map, IOError, identity]:
        assert identity(n) is n


def test_suppress():
    class TestException(Exception):
        pass

    def fail():
        raise TestException

    with pytest.raises(TypeError):
        with suppress('Not an exception'):
            assert False  # should not reach here

    with pytest.raises(TestException):
        fail()

    with suppress(TestException):
        fail()

    with suppress(ZeroDivisionError):
        42 / 0

    with pytest.raises(TestException):
        with suppress(ZeroDivisionError):
            fail()

    with suppress(ZeroDivisionError, TestException):
        fail()
        42 / 0



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.ftry - The Try Monad."""

from . import Monad, Monadic
from ..mixins import ContextManager, Ord


class Try(Monad, ContextManager, Ord):
    """The Either Monad by a different name

    Represents values/computations with two possibilities.

    >>> Success(42)
    Success(42)
    >>> Success([1, 2, 3])
    Success([1, 2, 3])
    >>> Failure('Error')
    Failure('Error')
    >>> Success(Failure('Error'))
    Success(Failure('Error'))
    >>> isinstance(Success(1), Try)
    True
    >>> isinstance(Failure(None), Try)
    True
    >>> saving = 100
    >>> broke = Failure('I am broke')
    >>> spend = lambda cost: broke if cost > saving else Success(saving - cost)
    >>> spend(90)
    Success(10)
    >>> spend(120)
    Failure('I am broke')
    >>> safe_div = lambda a, b: Failure(str(a) + '/0') if b == 0 else Success(a / b)
    >>> safe_div(12.0, 6)
    Success(2.0)
    >>> safe_div(12.0, 0)
    Failure('12.0/0')

    Bind operation with ``>>``

    >>> inc = lambda n: Success(n + 1) if type(n) is int else Failure('Type error')
    >>> Success(0)
    Success(0)
    >>> Success(0) >> inc
    Success(1)
    >>> Success(0) >> inc >> inc
    Success(2)
    >>> Success('zero') >> inc
    Failure('Type error')

    Comparison with ``==``, as long as they are the same type and what's
    wrapped inside are comparable.

    >>> Failure(42) == Failure(42)
    True
    >>> Success(42) == Success(42)
    True
    >>> Failure(42) == Success(42)
    False

    A :py:class:`Failure` is less than a :py:class:`Success`, or compare the two by
    the values inside if thay are of the same type.

    >>> Failure(42) < Success(42)
    True
    >>> Success(0) > Failure(100)
    True
    >>> Failure('Error message') > Success(42)
    False
    >>> Failure(100) > Failure(42)
    True
    >>> Success(-2) < Success(-1)
    True
    """
    def __init__(self, value):
        super(Try, self).__init__(value)
        if type(self) is Try:
            raise NotImplementedError('Please use Failure or Success instead')

    def bind(self, function):
        """The bind operation of :py:class:`Try`.

        Applies function to the value if and only if this is a
        :py:class:`Success`.
        """
        return self and function(self.value)

    def __repr__(self):
        """Customize Show."""
        fmt = 'Success({})' if self else 'Failure({})'
        return fmt.format(repr(self.value))

    # Customize Ord logic
    def __lt__(self, monad):
        """Override to handle special case: Success."""
        if not isinstance(monad, (Failure, Success)):
            fmt = "unorderable types: {} and {}'".format
            raise TypeError(fmt(type(self), type(monad)))
        if type(self) is type(monad):
            # same type, either both lefts or rights, compare against value
            return self.value < monad.value
        if monad:
            # self is Failure and monad is Success, left is less than right
            return True
        else:
            return False

    def match(self, failure_handler, right_handler=None):
        """
        Given two mapping functions (one from an Failure to a value, one from a Success to a
        Value), unwrap the value stored in this Try, apply the appropriate
        mapping function, and return the result.

        If the right_handler is None and self is an instance of Success, the wrapped value is returned
        """
        if isinstance(self, Failure):
            return failure_handler(self.value)

        if right_handler is None:
            return self.value

        return right_handler(self.value)

    def recover(self, recovery_fn):
        """
        "Recover" from a left value by applying a recovery_fn to the wrapped
        value and returning it in the case of a left value; otherwise, return
        the wrapped right value.
        """
        if isinstance(self, Failure):
            return recovery_fn(self.value)

        else:
            return self


class Failure(Try):
    """Failure of :py:class:`Try`."""
    def __bool__(self):
        # pylint: disable = no-self-use
        return False
    __nonzero__ = __bool__


class Success(Try):
    """Success of :py:class:`Try`."""


Try.unit = Monadic(Success)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.applicative - The Applicative Functor Class."""

from . import Functor


class Applicative(Functor):
    """The Applicative Functor Class.

    Defines the following functions:

    - ``unit`` which act as constructor, it's called ``pure`` in some context.
    """
    # pylint: disable = abstract-method, too-few-public-methods
    #: The unit.
    #:
    #: Maps a value to a value in this type.
    #: Also called ``pure`` or ``return`` depends on context.
    unit = NotImplemented



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""monad.utils - utility functions and values."""

from collections import Iterable


def succeeded(obj):
    """Return a Boolean that indicates if the value is an instance of smonad.types.ftry.Success

    >>> from smonad.types.ftry import Failure, Success
    >>> succeeded(Success(True))
    True
    >>> succeeded(Failure('fubar'))
    False
    """
    # import inside the function to avoid circular module dependencies
    from smonad.types.ftry import Try, Success
    assert isinstance(obj, Try)
    return isinstance(obj, Success)


def failed(obj):
    """Return a Boolean that indicates if the value is an instance of smonad.types.ftry.Failure

    >>> from smonad.types.ftry import Failure, Success
    >>> failed(Failure('shit is fucked up'))
    True
    >>> failed(Success('it worked!'))
    False
    """
    # import inside the function to avoid circular module dependencies
    from smonad.types.ftry import Try, Failure
    assert isinstance(obj, Try)
    return isinstance(obj, Failure)


# cannot wait for python 3.4, also do a type-check here
class SuppressContextManager(object):
    """Context manager class that suppress specified exceptions."""
    # pylint: disable = too-few-public-methods
    def __init__(self, *exceptions):
        # except accepts exception or expects exceptions in tuple. :p
        invalid_args = (
            not isinstance(ex, type) or not issubclass(ex, BaseException)
            for ex in exceptions)
        if any(invalid_args):
            raise TypeError('argument must be a subclass of BaseException')

        self.exceptions = exceptions

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        if isinstance(exc_val, self.exceptions):
            return True
        return False


def suppress(*exceptions):
    """Context manager that suppress specified exceptions.

    >>> with suppress(ZeroDivisionError):
    ...     42 / 0
    """
    return SuppressContextManager(*exceptions)


def compose(f, g):
    """Function composition.

    ``compose(f, g) -> f . g``

    >>> add_2 = lambda a: a + 2
    >>> mul_5 = lambda a: a * 5
    >>> mul_5_add_2 = compose(add_2, mul_5)
    >>> mul_5_add_2(1)
    7
    >>> add_2_mul_5 = compose(mul_5, add_2)
    >>> add_2_mul_5(1)
    15
    """
    # pylint: disable = invalid-name, star-args
    return lambda *args, **kwargs: f(g(*args, **kwargs))


def identity(a):
    """Identity function."""
    # pylint: disable = invalid-name
    return a


def ignore_exception_set(*exceptions):
    """Helper function for suppress."""
    to_be_ignored = set()
    for exception in exceptions:
        if not exception:
            continue
        if not isinstance(exception, Iterable):
            exception = (exception,)
        to_be_ignored |= set(exception)
    return to_be_ignored



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.decorators - helpful decorators."""

from functools import partial, wraps

from .exceptions import ExtractError
from .types import Null
from .types import Function
from .types import Monadic
from .types import Just, Nothing
from .types import Failure, Success
from .types import List
from .utils import ignore_exception_set, suppress


def function(callable_object):
    """Decorator that wraps a callabe into :class:`Function`.

    >>> to_int = function(int)
    >>> to_int('42')
    42
    >>> @function
    ... def puts(msg, times=1):
    ...     while times > 0:
    ...         print(msg)
    ...         times -= 1
    >>> puts('Hello, world', 2)
    Hello, world
    Hello, world
    """
    return Function(callable_object)


def monadic(callable_object):
    """Decorator that wraps a callabe into :class:`Monadic`."""
    return Monadic(callable_object)


def maybe(callable_object=None,
          predicate=None,
          nothing_on_value=Null,
          nothing_on_exception=Exception):
    """Transform a callable into a function returns a :py:class:`Maybe`.

    >>> parse_int = maybe(int)
    >>> parse_int(42)
    Just(42)
    >>> parse_int(42.0)
    Just(42)
    >>> parse_int('42')
    Just(42)
    >>> parse_int('invalid')
    Nothing

    >>> parse_pos = maybe(int, predicate=lambda i: i > 0)
    >>> parse_pos('42')
    Just(42)
    >>> parse_pos('-42')
    Nothing

    >>> parse_nonzero = maybe(int, nothing_on_value=0)
    >>> parse_nonzero('42')
    Just(42)
    >>> parse_nonzero('0')
    Nothing

    >>> @maybe(nothing_on_exception=ZeroDivisionError)
    ... def safe_div(a, b):
    ...     return a / b
    >>> safe_div(42.0, 2)
    Just(21.0)
    >>> safe_div(42, 0)
    Nothing

    When invoked, this new function returns the return value of decorated
    function, wrapped in a :py:class:`Maybe` monad.

    ``predicate`` should be a false value, or be set to a callable.
    The default is ``None``.

    ``nothing_on_value`` can be set to any object supporting comparison
    against return value of the original function.
    The default is ``Null``, which means no checking on the return value.

    ``nothing_on_exception`` can be a false value, a type of exception, or a
    tuple of exceptions.
    The default is ``Exception``, which will suppress most exceptions and
    return ``Nothing`` instead.


    The returned monad will be ``Nothing`` if

    - ``predicate`` is set, and ``predicate(result_from_decorated_function)``
      returns true value (not necessarily equal to ``True``)
    - ``nothing_on_value`` is set and the result from decorated function
      matches it, testing with ``==``
    - ``nothing_on_exception`` is set and a compatible exception has been
      caught, the exception will be suppressed in this case
    - exception ``ExtractError`` has been caught, when trying to extract value
      from ``Nothing``
    - any combination of the above

    Otherwise, the result will be wrapped in a :py:class:`Just`.
    """
    if callable_object is None:
        return partial(maybe,
                       predicate=predicate,
                       nothing_on_value=nothing_on_value,
                       nothing_on_exception=nothing_on_exception)

    exceptions = ignore_exception_set(ExtractError, nothing_on_exception)

    @wraps(callable_object)
    def wrapper(*args, **kwargs):
        """Monadic function wrapper for Maybe"""
        # pylint: disable = star-args
        with suppress(*exceptions):
            result = callable_object(*args, **kwargs)
            if nothing_on_value is not Null and result == nothing_on_value:
                return Nothing
            if predicate is not None and not predicate(result):
                return Nothing
            return Just(result)
        return Nothing
    return monadic(wrapper)


def failsafe(callable_object=None,
             predicate=None,
             failure_on_value=Null,
             failure_on_exception=Exception):
    """Transform a callable into a function returns an :py:class:`Try`.

    >>> parse_int = failsafe(int)
    >>> parse_int(42)
    Success(42)
    >>> parse_int(42.0)
    Success(42)
    >>> parse_int('42')
    Success(42)
    >>> parse_int('invalid')
    Failure(ValueError(...))

    >>> parse_pos = failsafe(int, predicate=lambda i: i > 0)
    >>> parse_pos('42')
    Success(42)
    >>> parse_pos('-42')
    Failure(-42)

    >>> parse_nonzero = failsafe(int, failure_on_value=0)
    >>> parse_nonzero('42')
    Success(42)
    >>> parse_nonzero('0')
    Failure(0)

    >>> @failsafe(failure_on_exception=ZeroDivisionError)
    ... def safe_div(a, b):
    ...     return a / b
    >>> safe_div(42.0, 2)
    Success(21.0)
    >>> safe_div(42, 0)
    Failure(ZeroDivisionError(...))

    When invoked, this new function returns the return value of decorated
    function, wrapped in an :py:class:`Either` monad.

    ``predicate`` should be a false value, or be set to a callable.
    The default is ``None``.

    ``failure_on_value`` can be set to any object supporting comparison against
    return value of the original function.
    The default is ``Null``, which means no checking on the return value.

    ``failure_on_exception`` should be a false value, or a type of exception,
    or a tuple of exceptions.
    The default is ``Exception``, which will suppress most exceptions and
    return ``Failure(exception)`` instead.

    The returned monad will be :py:class:`Failure` if

    - ``predicate`` is set, and ``predicate(result_from_decorated_function)``
      returns true value (not necessarily equal to ``True``)
    - ``failure_on_value`` is set and the result from decorated function matches
      it, testing with ``==``
    - ``failure_on_exception`` is set and a compatible exception has been caught,
      the exception will be suppressed in this case, and the value of
      exception will be wrapped in a :py:class:`Failure`
    - exception ``ExtractError`` has been caught, this could be the case, for
      example, trying to extract value from ``Nothing``
    - any combination of the above

    Otherwise, the result will be wrapped in a :py:class:`Success`.
    """
    if callable_object is None:
        return partial(failsafe,
                       predicate=predicate,
                       failure_on_value=failure_on_value,
                       failure_on_exception=failure_on_exception)

    exceptions = ignore_exception_set(failure_on_exception)

    @wraps(callable_object)
    def wrapper(*args, **kwargs):
        """Monadic function wrapper for Either"""
        try:
            result = callable_object(*args, **kwargs)
            if failure_on_value is not Null and result == failure_on_value:
                return Failure(result)
            if predicate is not None and not predicate(result):
                return Failure(result)
            return Success(result)
        except ExtractError as ex:
            monad = ex.monad
            if isinstance(monad, Failure):
                return ex.monad
            else:
                return Failure(ex)
        except tuple(exceptions) as ex:
            return Failure(ex)
    return monadic(wrapper)


def producer(function_or_generator=None,
             empty_on_exception=None):
    """Transform a callable into a producer that when called, returns ``List``.

    >>> @producer
    ... def double(a):
    ...     yield a
    ...     yield a
    >>> List(42) >> double
    List(42, 42)

    >>> @producer
    ... def times(a):
    ...     for b in List(1, 2, 3):
    ...         yield '{}x{}={}'.format(a, b, a * b)
    >>> List(1, 2) >> times
    List('1x1=1', '1x2=2', '1x3=3', '2x1=2', '2x2=4', '2x3=6')

    ``function_or_generator`` can be a function that returns an iterable, or a
    generator.

    ``empty_on_exception`` can be a false value, a type of exception, or a
    tuple of exceptions.
    The default is ``None``, which will not suppress all exceptions except
    ``ExtractError``, in which case, an empty :py:class:`List` will be
    returned.
    """
    if function_or_generator is None:
        return partial(producer, empty_on_exception=empty_on_exception)

    exceptions = ignore_exception_set(ExtractError, empty_on_exception)

    @wraps(function_or_generator)
    def wrapper(*args, **kwargs):
        """Monadic function wrapper for List"""
        # pylint: disable = star-args
        with suppress(*exceptions):
            return List.from_iterable(function_or_generator(*args, **kwargs))
        return List.zero
    return monadic(wrapper)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.identity - The Identity Monad."""

from . import Monad
from ..mixins import ContextManager, Ord


class Identity(Monad, ContextManager, Ord):
    """The Identity Monad.

    >>> Identity(42)
    Identity(42)
    >>> Identity([1, 2, 3])
    Identity([1, 2, 3])

    Comparison with ``==``, as long as what's wrapped inside are comparable.

    >>> Identity(42) == Identity(42)
    True
    >>> Identity(42) == Identity(24)
    False
    """
    def bind(self, function):
        return function(self.value)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Try
from smonad.types import List

testee = [
    Identity,
    Maybe,
    Try,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'monad' in metafunc.funcargnames:
        metafunc.parametrize('monad', testee, ids=ids)


def test_unit(monad):
    for n in test_range:
        # unit should act as a contructor
        assert isinstance(monad.unit(n), monad)


def test_join(monad):
    unit = monad.unit
    for n in test_range:
        # join should remove one level of monadic structure
        # m m a -> m a
        m = unit(n)
        assert unit(m).join() == m


def test_bind(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        m = unit(n)
        assert m.bind(add_1) == unit(n + 1)
        assert m.bind(double) == unit(n * 2)
        assert add_1(n).bind(double) == unit((n + 1) * 2)


def test_bind_operator(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        m = unit(n)
        assert m >> add_1 == unit(n + 1)
        assert m >> double == unit(n * 2)
        assert add_1(n) >> double == unit((n + 1) * 2)


def test_bind_operator_type_check(monad):
    unit = monad.unit
    non_callable = object()
    with pytest.raises(TypeError):
        unit(1) >> non_callable


def test_reversed_bind_operator(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        m = unit(n)
        assert add_1 << m == m >> add_1
        assert double << m == m >> double
        assert double << add_1(n) == add_1(n) >> double


def test_reversed_bind_operator_type_check(monad):
    unit = monad.unit
    non_callable = object()
    with pytest.raises(TypeError):
        non_callable << unit(1)


def test_chain_bind_operator(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        m = unit(n)
        assert m >> add_1 >> add_1 == unit(n + 1 + 1)
        assert m >> double >> double == unit(n * 2 * 2)
        assert m >> add_1 >> double == unit((n + 1) * 2)
        assert m >> double >> add_1 == unit(n * 2 + 1)


def test_chain_reversed_bind_operator(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        m = unit(n)
        # NOTE: because there is no way to redefine the evaluation direction
        # of operators in python, parentheses are needed, to test the chain
        # action of reversed bind operator.
        assert add_1 << (add_1 << m) == m >> add_1 >> add_1
        assert double << (double << m) == m >> double >> double
        assert add_1 << (double << m) == m >> double >> add_1
        assert double << (add_1 << m) == m >> add_1 >> double


def test_monad_law_left_identity(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        # unit n >>= f == f n
        f = unit
        assert unit(n) >> f == f(n)
        f = add_1
        assert unit(n) >> f == f(n)
        f = double
        assert unit(n) >> f == f(n)


def test_monad_law_right_identity(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        # m >>= unit == m
        m = unit(n)
        assert m >> unit == m
        m = add_1(n)
        assert m >> unit == m
        m = double(n)
        assert m >> unit == m


def test_monad_law_associativity(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        # m >>= (\x -> k x >>= h)  ==  (m >>= k) >>= h
        m = unit(n)
        k = add_1
        h = double
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = lambda a: unit(str(a))
        h = lambda a: unit(hash(a))
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.monad - The Monad Class."""

from . import Applicative
from ..utils import identity


class Unit(object):
    """Descriptor that always return the owner monad, used for ``unit``."""
    # pylint: disable = too-few-public-methods
    def __get__(self, instance, cls):
        """Returns the owner monad."""
        return cls


class Monad(Applicative):
    """The Monad Class.

    Implements bind operator ``>>`` and inverted bind operator ``<<`` as
    syntactic sugar.  It is equivalent to ``(>>=)`` and ``(=<<)`` in haskell,
    not to be confused with ``(>>)`` and ``(<<)`` in haskell.

    As python treats assignments as statements, there is no way we can
    overload ``>>=`` as a chainable bind, be it directly overloaded through
    ``__irshift__``, or derived by python itself through ``__rshift__``.

    The default implementations of ``bind``, ``fmap`` and ``join`` are mutual
    recursive, subclasses should at least either overload ``bind``, or
    ``fmap`` and ``join``, or all of them for better performance.
    """
    # Bind Operators
    def __rshift__(self, function):
        """The bind operator ``>>``"""
        if not callable(function):
            return NotImplemented
        return self.bind(function)

    def __rlshift__(self, function):
        """The inverted bind operator ``<<``"""
        if not callable(function):
            return NotImplemented
        return self.bind(function)

    def __repr__(self):
        return '{cls}({value})'.format(
            cls=type(self).__name__, value=repr(self.value))

    def bind(self, function):
        """The bind operation.

        ``function`` is a function that maps from the underlying value to a
        monadic type, something like signature ``f :: a -> M a`` in haskell's
        term.

        The default implementation defines ``bind`` in terms of ``fmap`` and
        ``join``.
        """
        return self.fmap(function).join()

    def fmap(self, function):
        """The fmap operation.

        The default implementation defines ``fmap`` in terms of ``bind`` and
        ``unit``.
        """
        # pylint: disable = star-args
        return self.bind(
            lambda *args, **kwargs: self.unit(function(*args, **kwargs))
        )

    def join(self):
        """The join operation.

        The default implementation defines ``join`` in terms of ``bind`` and
        ``identity`` function.
        """
        return self.bind(identity)

    #: The ``unit`` of monad.
    unit = Unit()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.exceptions - custom exceptions."""


class ExtractError(Exception):
    """Raised when failed to extract value from monad."""
    def __init__(self, monad):
        super(ExtractError, self).__init__(
            'cannot extract value from {}'.format(repr(monad)))
        self.monad = monad



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.null - The Null type."""


class Null(object):
    """Null represents nothing."""
    # pylint: disable = too-few-public-methods
    def __repr__(self):
        return 'Null'

    def __call__(self):
        return self


# pylint: disable = invalid-name
#: The Null object.
Null = Null()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.maybe - The Maybe Monad."""

from . import MonadPlus, Null
from ..mixins import ContextManager, Ord


class Maybe(MonadPlus, ContextManager, Ord):
    """The Maybe Monad.

    Representing values/computations that may fail.

    >>> Just(42)
    Just(42)
    >>> Just([1, 2, 3])
    Just([1, 2, 3])
    >>> Just(Nothing)
    Just(Nothing)
    >>> Just(Just(2))
    Just(Just(2))
    >>> isinstance(Just(1), Maybe)
    True
    >>> isinstance(Nothing, Maybe)
    True
    >>> saving = 100
    >>> spend = lambda cost: Nothing if cost > saving else Just(saving - cost)
    >>> spend(90)
    Just(10)
    >>> spend(120)
    Nothing
    >>> safe_div = lambda a, b: Nothing if b == 0 else Just(a / b)
    >>> safe_div(12.0, 6)
    Just(2.0)
    >>> safe_div(12.0, 0)
    Nothing

    Bind operation with ``>>``

    >>> inc = lambda n: Just(n + 1) if isinstance(n, int) else Nothing
    >>> Just(0)
    Just(0)
    >>> Just(0) >> inc
    Just(1)
    >>> Just(0) >> inc >> inc
    Just(2)
    >>> Just('zero') >> inc
    Nothing

    Comparison with ``==``, as long as what's wrapped inside are comparable.

    >>> Just(42) == Just(42)
    True
    >>> Just(42) == Nothing
    False
    >>> Nothing == Nothing
    True
    """
    @classmethod
    def from_value(cls, value):
        """Wraps ``value`` in a :class:`Maybe` monad.

        Returns a :class:`Just` if the value is evaluated as true.
        :data:`Nothing` otherwise.
        """
        return cls.unit(value) if value else Nothing

    def bind(self, function):
        """The bind operation of :class:`Maybe`.

        Applies function to the value if and only if this is a :class:`Just`.
        """
        return Nothing if self is Nothing else function(self.value)

    def __bool__(self):
        return self is not Nothing
    __nonzero__ = __bool__

    def __repr__(self):
        """Customized Show."""
        if self is Nothing:
            return 'Nothing'
        else:
            return 'Just({})'.format(repr(self.value))

    def __iter__(self):
        if self is not Nothing:
            yield self.value

    # Customized Ord logic
    def __lt__(self, monad):
        """Override to handle special case: Nothing."""
        if self is Nothing and monad is Nothing:
            return False
        elif self is Nothing:
            # Nothing is less than something.
            return True
        elif monad is Nothing:
            # self is not Nothing and monad is Nothing here
            return False
        else:
            # Test normally
            return super(Maybe, self).__lt__(monad)

    # MonadPlus operation
    def plus(self, monad):
        return self or monad


# pylint: disable = invalid-name
Just = Maybe
#: The :class:`Maybe` that represents nothing, a singleton, like ``None``.
Nothing = Maybe(Null)
Maybe.zero = Nothing
# pylint: enable = invalid-name



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.actions import ftry
from smonad.decorators import failsafe
from smonad.exceptions import ExtractError
from smonad.types import Try, Failure, Success


test_range = range(-100, 100)
unit = Try.unit

error = Failure('Error')


def add_1(n):
    if isinstance(n, int):
        return unit(n + 1)
    else:
        return error


def double(n):
    if isinstance(n, int):
        return unit(n * 2)
    else:
        return error


def fail(n):
    return error


def test_local_helper_function_add_one():
    for n in test_range:
        assert add_1(n) == unit(n + 1)
    assert add_1('1') is error


def test_local_helper_function_double():
    for n in test_range:
        assert double(n) == unit(n * 2)
    assert double('1') is error


def test_local_helper_function_fail():
    for n in test_range:
        assert fail(n) is error


def test_fmap_functor_laws():
    identity = lambda a: a
    f = lambda a: a + 1
    g = lambda a: a * 2
    f_g = lambda n: f(g(n))

    for n in test_range:
        ft = unit(n)
        # fmap id == id
        assert ft.fmap(identity) == identity(ft)
        # fmap (f . g) == fmap f . fmap g
        assert ft.fmap(f_g) == ft.fmap(g).fmap(f)

    value = 42
    l = Failure('Something wrong.')
    r = Success(value)
    assert l.fmap(f) is l
    assert r.fmap(f) == Success(f(42))


def test_unit():
    assert type(unit(42)) is Success


def test_ftry_is_abstract():
    with pytest.raises(NotImplementedError):
        Try(42)


def test_compare():
    for n in test_range:
        assert Failure(n) == Failure(n)
        assert Success(n) == Success(n)
        assert Failure(n) != Success(n)


def test_ordering():
    with pytest.raises(TypeError):
        Failure(1) < 1

    with pytest.raises(TypeError):
        Success(1) < 1

    for n in test_range:
        assert (Failure(n) < Failure(n)) is False
        assert Failure(n) > Failure(n - 1)
        assert Failure(n) < Failure(n + 1)
        assert (Success(n) < Success(n)) is False
        assert Success(n) > Success(n - 1)
        assert Success(n) < Success(n + 1)
        assert Failure(n) < Success(n)


def test_as_context_manager():
    for n in test_range:
        with pytest.raises(ExtractError):
            with unit(n) >> double >> fail >> double as result:
                assert False
                assert result

    with pytest.raises(ExtractError):
        with error as n:
            assert False

    with pytest.raises(ExtractError):
        with double(n) as result:
            with error as n:
                assert False

    with pytest.raises(ExtractError):
        with double(n) as result, error as n:
            assert False


def test_bool():
    assert bool(Failure(True)) is False
    assert bool(Success(False)) is True
    for n in test_range:
        assert bool(Failure(n)) is False
        assert bool(Success(n)) is True
        assert bool(unit(n)) is True


def test_bind():
    assert error.bind(add_1) is error
    for n in test_range:
        m = unit(n)
        assert m.bind(fail) is error


def test_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> fail is error
        assert fail(n) >> add_1 is error


def test_reversed_bind_operator():
    for n in test_range:
        m = unit(n)
        assert fail << m is error
        assert add_1 << fail(n) is error


def test_chain_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> fail >> add_1 == error
        assert m >> add_1 >> fail == error
        assert m >> fail >> double == error
        assert m >> double >> fail == error


def test_monad_law_left_identity():
    for n in test_range:
        # unit n >>= f == f n
        f = fail
        assert unit(n) >> f == f(n)


def test_monad_law_right_identity():
    for n in test_range:
        # m >>= unit == m
        assert error >> unit == error


def test_monad_law_associativity():
    for n in test_range:
        # m >>= (\x -> k x >>= h)  ==  (m >>= k) >>= h
        m = unit(n)
        k = add_1
        h = fail
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = fail
        h = double
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = fail
        h = fail
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h


def test_ftry_action():
    inc = lambda n: n + 1
    dec = lambda n: n - 1

    act = ftry(inc)
    assert act(Failure(1)) == 2
    assert act(Success(1)) == 1

    act = ftry(failure_handler=inc, success_handler=dec)
    assert act(Failure(1)) == 2
    assert act(Success(1)) == 0


def test_ftry_action_with_incompatible_type():
    inc = lambda n: n + 1

    act = ftry(inc)
    assert act(Failure(1)) == 2

    with pytest.raises(TypeError):
        act(1)


def test_failsafe_decorator():
    @failsafe
    def div(a, b):
        return a / b

    assert div(42, 21) == unit(2)
    assert isinstance(div(42, 0), Failure)


def test_failsafe_decorator_catch_extract_error():
    @failsafe(failure_on_exception=None)
    def wrong():
        with fail(1) as result:
            assert result is False  # should not reach here

    assert wrong() == error

    @failsafe(failure_on_exception=None)
    def wrong():
        raise ExtractError('not a left')

    assert isinstance(wrong(), Failure)


def test_failsafe_decorator_with_predicate():
    @failsafe(predicate=bool)
    def truth(x):
        return x

    assert truth(42) == unit(42)
    assert truth(None) == Failure(None)
    assert add_1(0) >> truth == unit(1)
    assert add_1(-1) >> truth == Failure(0)
    assert truth(False) >> double == Failure(False)
    assert double([]) >> truth == error


def test_failsafe_decorator_with_value():
    @failsafe(failure_on_value=None)
    def truth(x):
        return x

    assert truth(42) == unit(42)
    assert truth('') == unit('')
    assert truth(0) == unit(0)
    assert truth(False) == unit(False)
    assert truth(None) == Failure(None)


def test_failsafe_decorator_combined():
    @failsafe(predicate=bool, failure_on_value=42)
    def wrap(x):
        return x

    assert wrap(True) == Success(True)
    assert wrap(False) == Failure(False)
    assert wrap('something') == Success('something')
    assert wrap('') == Failure('')
    assert wrap([False]) == Success([False])
    assert wrap([]) == Failure([])
    assert wrap(1) == Success(1)
    assert wrap(0) == Failure(0)
    assert wrap(None) == Failure(None)
    assert wrap(42) == Failure(42)


def test_failsafe_decorator_none_exception():
    @failsafe(failure_on_exception=None)
    def div(a, b):
        return a / b

    with pytest.raises(ZeroDivisionError):
        div(42, 0)


def test_failsafe_decorator_empty_seq_exception():
    for empty in ([], tuple(), set()):
        @failsafe(failure_on_exception=empty)
        def div(a, b):
            return a / b

        with pytest.raises(ZeroDivisionError):
            div(42, 0)


def test_failsafe_decorator_specific_exception():
    @failsafe(failure_on_exception=ZeroDivisionError)
    def div(a, b):
        return a / b

    assert isinstance(div(42, 0), Failure)


def test_failsafe_decorator_specific_exception_tuple():
    @failsafe(failure_on_exception=(IOError, ZeroDivisionError))
    def div(a, b):
        if a < 0:
            raise IOError
        return a / b

    assert isinstance(div(42, 0), Failure)
    assert isinstance(div(-42, 2), Failure)


def test_match_failure_only():
    assert Failure(0).match(lambda v: v + 1) == 1
    assert Success(10).match(lambda v: v + 1) == 10


def test_match_failure_and_success():
    assert Failure(0).match(lambda v: v + 1, lambda v: v / 2) == 1
    assert Success(10).match(lambda v: v + 1, lambda v: v / 2) == 5


def test_recover():
    result = Failure(0).recover(lambda v: v + 1)
    assert result == 1
    
    result = Success(10).recover(lambda v: v + 1)
    assert isinstance(result, Success)
    assert result.value == 10




from smonad.types.ftry import Success, Failure
from smonad import actions
import pytest


def test_attempt_failure():
    result = actions.attempt(lambda: 1 / 0, exception=ZeroDivisionError)
    assert isinstance(result, Failure)
    assert isinstance(result.value, ZeroDivisionError)
    assert result.value.message == "integer division or modulo by zero"


def test_attempt_failure_catch_all():
    result = actions.attempt(lambda: 1 / 0)
    assert isinstance(result, Failure)
    assert isinstance(result.value, ZeroDivisionError)
    assert result.value.message == "integer division or modulo by zero"


def test_attempt_failure_wrong_exception():
    with pytest.raises(ZeroDivisionError):
        actions.attempt(lambda: 1 / 0, exception=ValueError)


def test_attempt():
    result = actions.attempt(lambda: 1 / 1)
    assert isinstance(result, Success)
    assert result.value == 1





# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.lazysequence - a sequence type with lazy evaluation."""

from collections import Sequence
from functools import total_ordering
from itertools import islice

from ..utils import suppress


@total_ordering
class LazySequence(Sequence):
    """Sequence with lazy evaluation.

    >>> from itertools import count
    >>> seq = LazySequence(count())
    >>> seq[1]
    1
    >>> list(seq[3:5])
    [3, 4]
    >>> list(seq[:20:2])
    [0, 2, 4, 6, 8, 10, 12, 14, 16, 18]
    """
    def __init__(self, iterable):
        self.iterable = iter(iterable)
        self.items = []

    def __getitem__(self, index):
        if isinstance(index, slice):
            iterable = islice(self, index.start, index.stop, index.step)
            return self.__class__(iterable)

        if self.iterable is None:
            # that's all we have
            return self.items[index]

        item_numbers = len(self.items)
        finished = object()

        # get until enough
        while index < 0 or index >= item_numbers:
            next_item = next(self.iterable, finished)
            if next_item is finished:
                self.iterable = None
                break
            self.items.append(next_item)
            item_numbers += 1

        return self.items[index]

    def __len__(self):
        return len(self.strict.items)

    def __eq__(self, other):
        if self is other:
            return True
        elif isinstance(other, type(self)):
            return self.strict.items == other.strict.items
        else:
            return NotImplemented

    def __lt__(self, other):
        if self is other:
            return False
        elif isinstance(other, type(self)):
            return self.strict.items < other.strict.items
        else:
            fmt = "unorderable types: {} and {}'".format
            raise TypeError(fmt(type(self), type(other)))

    @property
    def strict(self):
        """Proxy to self that forces evaluation when accessed."""
        if self.iterable is not None:
            # force consume all items from self.iterable first
            with suppress(IndexError):
                id(self[-1])
        return self



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types - types"""

from .null import Null
from .lazysequence import LazySequence
from .functor import Functor
from .applicative import Applicative
from .function import Function
from .monadic import Monadic
from .monad import Monad
from .monadplus import MonadPlus
from .identity import Identity
from .maybe import Maybe, Just, Nothing
from .ftry import Try, Failure, Success
from .list import List

__all__ = [
    'Null',
    'LazySequence',
    'Functor',
    'Applicative',
    'Function',
    'Monadic',
    'Monad',
    'MonadPlus',
    'Identity',
    'Maybe',
    'Just',
    'Nothing',
    'Try',
    'Failure',
    'Success',
    'List',
]



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.decorators import producer
from smonad.types import List

test_range = range(-100, 100)
unit = List.unit
zero = List.zero


def add_1(n):
    return unit(n + 1)


def double(n):
    return unit(n, n)


def empty(n):
    return zero


def test_compare():
    assert zero == List()


def test_ording():
    for n in test_range:
        assert zero < List(n)


def test_bool():
    assert bool(zero) is False
    for n in test_range:
        assert bool(unit(n)) is True


def test_from_iterable():
    fi = List.from_iterable
    assert fi(range(3)) == unit(0, 1, 2)
    assert fi([3, 2, 1]) == unit(3, 2, 1)
    assert fi(n for n in [3, 5, 8]) == unit(3, 5, 8)


def test_len():
    for n in range(100):
        m = List.from_iterable(range(n))
        assert len(m) == n


def test_getitem():
    rg = range(100)
    m = List.from_iterable(rg)
    for n in rg:
        assert m[n] == n


def test_bind():
    for n in test_range:
        m = unit(n)
        assert m.bind(add_1) == unit(n + 1)
        assert m.bind(double) == unit(n, n)
        assert add_1(n).bind(double) == unit((n + 1), (n + 1))

        assert m.bind(add_1).bind(empty) == zero
        assert zero.bind(add_1) == zero
        assert m.bind(double).bind(empty) == zero
        assert zero.bind(double) == zero


def test_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> add_1 == unit(n + 1)
        assert m >> double == unit(n, n)
        assert add_1(n) >> double == unit((n + 1), (n + 1))

        assert zero >> add_1 == zero
        assert zero >> double == zero
        assert m >> empty == zero


def test_reversed_bind_operator():
    for n in test_range:
        m = unit(n)
        assert add_1 << m == m >> add_1
        assert double << m == m >> double
        assert double << add_1(n) == add_1(n) >> double

        assert add_1 << zero == zero >> add_1
        assert double << zero == zero >> double
        assert empty << m == m >> empty


def test_chain_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> add_1 >> add_1 == unit(n + 1 + 1)
        assert m >> double >> double == unit(n, n, n, n)
        assert m >> add_1 >> double == unit(n + 1, n + 1)
        assert m >> double >> add_1 == unit(n + 1, n + 1)
        assert m >> add_1 >> empty == zero
        assert m >> double >> empty == zero
        assert m >> empty >> add_1 == zero
        assert m >> empty >> double == zero


def test_monad_law_left_identity():
    for n in test_range:
        # unit n >>= f == f n
        f = unit
        assert unit(n) >> f == f(n)
        f = add_1
        assert unit(n) >> f == f(n)
        f = double
        assert unit(n) >> f == f(n)
        f = empty
        assert unit(n) >> f == f(n)


def test_monad_law_right_identity():
    for n in test_range:
        # m >>= unit == m
        m = unit(n)
        assert m >> unit == m
        m = add_1(n)
        assert m >> unit == m
        m = double(n)
        assert m >> unit == m
        m = empty(n)
        assert m >> unit == m


def test_monad_law_associativity():
    for n in test_range:
        # m >>= (\x -> k x >>= h)  ==  (m >>= k) >>= h
        m = unit(n)
        k = add_1
        h = double
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = add_1
        h = empty
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = empty
        h = double
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = empty
        h = empty
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h


def test_plus_type_check():
    assert unit(1).plus(1) == NotImplemented
    with pytest.raises(TypeError):
        unit(1) + 1


def test_list_is_lazy():
    from itertools import count
    # no need to use assert here, if LazySequence is not lazy, execution will
    # never stop in for statement.
    infinite = List.from_iterable(count())
    for n in infinite:
        if n == 42:
            break

    def never_stop(n):
        return infinite

    # this is okay because zero binds something yields zero immediately
    # without evaluating that something.
    assert zero >> never_stop >> double == zero


def test_list_comprehension():
    a = [n for o in unit(1, 2, 3) for m in double(o) for n in add_1(m)]
    assert a == [2, 2, 3, 3, 4, 4]
    a = [n for o in unit(1, 2, 3) for m in empty(o) for n in add_1(m)]
    assert a == []


def test_producer_decorator():
    @producer
    def range_iter():
        return test_range

    for m, n in zip(range_iter(), test_range):
        assert m == n

    @producer
    def list_iter():
        return list(test_range)

    for m, n in zip(list_iter(), test_range):
        assert m == n

    @producer
    def generator():
        yield 1
        yield 2
        yield 3

    assert generator() == List(1, 2, 3)


def test_producer_empty_on_exception():
    class Error(Exception):
        pass

    @producer(empty_on_exception=None)
    def fail():
        raise Error

    with pytest.raises(Error):
        fail()

    @producer(empty_on_exception=Error)
    def empty():
        raise Error

    assert empty() == unit()



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad - a functional library"""

__version__ = (0, 6)

VERSION = '%d.%d' % __version__

__all__ = [
]



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Try

testee = [
    Identity,
    Maybe,
    Try,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'monad' in metafunc.funcargnames:
        metafunc.parametrize('monad', testee, ids=ids)


def test_as_context_manager(monad):
    unit = monad.unit
    add_1 = lambda n: unit(n + 1)
    double = lambda n: unit(n * 2)
    for n in test_range:
        with unit(n) as num:
            with add_1(num) as n_1:
                with double(n_1) as result:
                    assert result == (n + 1) * 2

    for n in test_range:
        with unit(n) as num, double(num) as n2, add_1(n2) as result:
            assert result == n * 2 + 1

    for n in test_range:
        with unit(n) >> double >> add_1 >> double as result:
            assert result == (n * 2 + 1) * 2



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.monadic - The Monadic Fuction Wrapper."""

from . import Function


class Monadic(Function):
    """The Monadic Function Wrapper.

    Implements Kleisli composition operators ``>>`` and ``<<``.  It is
    equivalent to ``(>=>)`` and ``(<=<)`` in haskell.
    """
    # pylint: disable = too-few-public-methods
    def __lshift__(self, monad):
        return monad >> self

    def __rshift__(self, monadic):
        """Left-to-right Kleisli composition of monads. ``>>``"""
        if not callable(monadic):
            return NotImplemented
        # pylint: disable = star-args
        composed = lambda *args, **kwargs: self(*args, **kwargs) >> monadic
        return self.__class__(composed)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.function - The Function Wrapper."""

from ..utils import compose


class Function(object):
    """The Function Wrapper.

    Support function composition via ``*`` operator.

    >>> add_1 = Function(lambda n: n + 1)
    >>> inc = add_1 * int
    >>> inc('42')
    43

    Support function piping via ``|`` operator.

    >>> inc2 = int | add_1 | add_1 | str
    >>> inc2('42')
    '44'
    """
    # pylint: disable = too-few-public-methods
    def __init__(self, callable_object):
        self.function = callable_object
        # just copy these, functools.wraps does too much
        for attr in ('__module__', '__name__', '__doc__'):
            setattr(self, attr, getattr(callable_object, attr, None))

    def __call__(self, *args, **kwargs):
        return self.function(*args, **kwargs)

    def __mul__(self, other):
        return self.__class__(compose(self.function, other))

    def __ror__(self, other):
        return self.__mul__(other)

    def __rmul__(self, other):
        return self.__class__(compose(other, self.function))

    def __or__(self, other):
        return self.__rmul__(other)



# -*- coding: utf-8 -*-
import sys
from os import path
from distutils.core import setup
from smonad import VERSION
from smonad import __doc__ as DESCRIPTION

if sys.version_info < (2, 7):
    sys.exit('smonad requires Python 2.7 or higher')

ROOT_DIR = path.abspath(path.dirname(__file__))
sys.path.insert(0, ROOT_DIR)

LONG_DESCRIPTION = open(path.join(ROOT_DIR, 'README.rst')).read()

setup(
    name='smonad',
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    author='Bryan W. Berry',
    author_email='bryan.berry@gmail.com',
    url='https://github.com/bryanwb/smonad/',
    download_url=(
        'https://github.com/bryanwb/smonad/archive/%s.zip' % VERSION),
    packages=['smonad', 'smonad.types'],
    license='BSD-New',
)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.actions - useful monadic actions."""

from .decorators import function, monadic
from .types import Try, Failure, Success
from .types import Just, Nothing
from .utils import identity
import StringIO
import sys
import traceback


def _get_stacktrace():
    t, _, tb = sys.exc_info()
    f = StringIO.StringIO()
    traceback.print_tb(tb, None, f)
    stacktrace = f.getvalue()
    return stacktrace


def attempt(callable, exception=None):
    """Call callable and wrap it with appropriate Failure or Success

    If calling the callable raises an exception, wrap the exception in a Failure.
    If the keyword argument ``exception`` is not None, only wrap
    the specified exception. Raise all other exceptions.
    The stacktrace related to the exception is added to the wrapped exception
    as the `stractrace` property

    If the calling the callable does not raise an exception, return Success 
    wrapping the value
    """
    if exception is None:
        catch_exception = Exception
    else:
        catch_exception = exception

    try:
        value = callable()
        return Success(value)
    except catch_exception as e:
        stacktrace = _get_stacktrace()
        e.stacktrace = stacktrace
        return Failure(e)
    
    
@function
def ftry(failure_handler, success_handler=identity):
    """Case analysis for ``Try``.

    This function is named ftry instead of try to avoid collision with the ``try`` keyword.

    Returns a function that when called with a value of type ``Try``,
    applies either ``failure_handler`` or ``success_handler`` to that value
    depending on the type of it.  If an incompatible value is passed, a
    ``TypeError`` will be raised.

    >>> def log(v):
    ...     print('Got Failure({})'.format(v))
    >>> logger = ftry(failure_handler=log)
    >>> logger(Failure(1))
    Got Failure(1)
    >>> logger(Success(1))
    1
    >>> def inc(v):
    ...     return v + 1
    >>> act = ftry(log, inc)
    >>> [act(v) for v in (Failure(0), Success(1), Failure(2), Success(3))]
    Got Failure(0)
    Got Failure(2)
    [None, 2, None, 4]
    """
    @function
    def analysis(a_try):
        """Apply handler functions based on value."""
        aug_type = type(a_try)
        if not issubclass(aug_type, Try):
            raise TypeError(
                'applied either on incompatible type: %s' % aug_type)
        if issubclass(aug_type, Failure):
            return failure_handler(a_try.value)
        assert issubclass(aug_type, Success)
        return success_handler(a_try.value)
    return analysis


@function
def tryout(*functions):
    """Combine functions into one.

    Returns a monadic function that when called, will try out functions in
    ``functions`` one by one in order, testing the result, stop and return
    with the first value that is true or the last result.

    >>> zero = lambda n: 'zero' if n == 0 else False
    >>> odd = lambda n: 'odd' if n % 2 else False
    >>> even = lambda n: 'even' if n % 2 == 0 else False
    >>> test = tryout(zero, odd, even)
    >>> test(0)
    'zero'
    >>> test(1)
    'odd'
    >>> test(2)
    'even'
    """
    @monadic
    def trying(*args, **kwargs):
        """Monadic function that try out functions in order."""
        last = None
        for func in functions:
            last = func(*args, **kwargs)
            if last:
                break
        return last
    return trying


@monadic
def first(sequence, default=Nothing, predicate=None):
    """Iterate over a sequence, return the first ``Just``.

    If ``predicate`` is provided, ``first`` returns the first item that
    satisfy the ``predicate``, the item will be wrapped in a :class:`Just` if
    it is not already, so that the return value of this function will be an
    instance of :class:`Maybe` in all circumstances.
    Returns ``default`` if no satisfied value in the sequence, ``default``
    defaults to :data:`Nothing`.

    >>> from smonad.types import Just, Nothing
    >>> first([Nothing, Nothing, Just(42), Nothing])
    Just(42)
    >>> first([Just(42), Just(43)])
    Just(42)
    >>> first([Nothing, Nothing, Nothing])
    Nothing
    >>> first([])
    Nothing
    >>> first([Nothing, Nothing], default=Just(2))
    Just(2)
    >>> first([False, 0, True], predicate=bool)
    Just(True)
    >>> first([False, 0, Just(1)], predicate=bool)
    Just(1)
    >>> first([False, 0, ''], predicate=bool)
    Nothing
    >>> first(range(100), predicate=lambda x: x > 40 and x % 2 == 0)
    Just(42)
    >>> first(range(100), predicate=lambda x: x > 100)
    Nothing

    This is basically a customized version of ``msum`` for :class:`Maybe`,
    a separate function like this is needed because there is no way to write a
    generic ``msum`` in python that cab be evaluated in a non-strict way.
    The obvious ``reduce(operator.add, sequence)``, albeit beautiful, is
    strict, unless we build up the sequence with generator expressions
    in-place.

    Maybe (pun intended!) implemented as ``MonadOr`` instead of ``MonadPlus``
    might be more semantically correct in this case.
    """
    if predicate is None:
        predicate = lambda m: m and isinstance(m, Just)

    for item in sequence:
        if predicate(item):
            if not isinstance(item, Just):
                item = Just(item)
            return item
    return default


# As decorators function and monadic turn decorated functions into Function
# and Monadic instance objects, respectively, doctest will ignore docstring in
# them, the following adds those docstring into testsuite back again,
# explicitly.
__test__ = {
    'attempt': ftry.__doc__,
    'ftry': ftry.__doc__,
    'tryout': tryout.__doc__,
    'first': first.__doc__,
}



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.actions import first
from smonad.decorators import maybe
from smonad.exceptions import ExtractError
from smonad.types import Maybe, Just, Nothing

test_range = range(-100, 100)
unit = Maybe.unit


def add_1(n):
    if isinstance(n, int):
        return unit(n + 1)
    else:
        return Nothing


def double(n):
    if isinstance(n, int):
        return unit(n * 2)
    else:
        return Nothing


def fail(n):
    return Nothing


def test_local_helper_function_add_one():
    for n in test_range:
        assert add_1(n) == unit(n + 1)
    assert add_1('1') is Nothing


def test_local_helper_function_double():
    for n in test_range:
        assert double(n) == unit(n * 2)
    assert double('1') is Nothing


def test_local_helper_function_fail():
    for n in test_range:
        assert fail(n) is Nothing


def test_type():
    assert unit(1) == Just(1) == Maybe(1)
    assert Nothing != unit(1)
    assert type(unit(1)) == type(Just(1)) == type(Maybe(1)) == type(Nothing)


def test_compare():
    assert Nothing == Nothing
    for n in test_range:
        assert unit(n) != Nothing
        assert unit(n) is not Nothing


def test_ordering():
    assert (Nothing < Nothing) is False
    assert (Nothing > Nothing) is False
    for n in test_range:
        assert (Nothing > unit(n)) is False
        assert (unit(n) < Nothing) is False


def test_as_context_manager():
    for n in test_range:
        with pytest.raises(ExtractError):
            with unit(n) >> double >> fail >> double as result:
                assert False
                assert result

    with pytest.raises(ExtractError):
        with Nothing as n:
            assert False

    with pytest.raises(ExtractError):
        with double(n) as result:
            with Nothing as n:
                assert False

    with pytest.raises(ExtractError):
        with double(n) as result, Nothing as n:
            assert False


def test_bool():
    assert bool(Nothing) is False
    for n in test_range:
        assert bool(unit(n)) is True


def test_from_value():
    false_v = [False, None, 0, 0.0, (), [], {}, '', set(), frozenset()]
    for v in false_v:
        assert Maybe.from_value(v) is Nothing

    true_v = [True, 1, 1.0, (0,), [0], {0: 0}, '0', set('0'), frozenset('0')]
    for v in true_v:
        assert Maybe.from_value(v) == unit(v)


def test_as_iterator():
    for n in test_range:
        for i in unit(n):
            assert i == n

        assert list(unit(n)) == [n]


def test_bind():
    assert Nothing.bind(add_1) is Nothing
    for n in test_range:
        m = unit(n)
        assert m.bind(fail) is Nothing


def test_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> fail is Nothing
        assert fail(n) >> add_1 is Nothing


def test_reversed_bind_operator():
    for n in test_range:
        m = unit(n)
        assert fail << m is Nothing
        assert add_1 << fail(n) is Nothing


def test_chain_bind_operator():
    for n in test_range:
        m = unit(n)
        assert m >> fail >> add_1 == Nothing
        assert m >> add_1 >> fail == Nothing
        assert m >> fail >> double == Nothing
        assert m >> double >> fail == Nothing


def test_monad_law_left_identity():
    for n in test_range:
        # unit n >>= f == f n
        f = fail
        assert unit(n) >> f == f(n)


def test_monad_law_right_identity():
    for n in test_range:
        # m >>= unit == m
        assert Nothing >> unit == Nothing


def test_monad_law_associativity():
    for n in test_range:
        # m >>= (\x -> k x >>= h)  ==  (m >>= k) >>= h
        m = unit(n)
        k = add_1
        h = fail
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = fail
        h = double
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h
        k = fail
        h = fail
        assert m >> (lambda x: k(x) >> h) == (m >> k) >> h


def test_maybe_decorator():
    @maybe
    def div(a, b):
        return a / b

    assert div(42, 21) == unit(2)
    assert div(42, 0) is Nothing


def test_maybe_decorator_with_predicate():
    @maybe(predicate=bool)
    def truth(x):
        return x

    assert truth(42) == unit(42)
    assert truth(None) is Nothing
    assert add_1(0) >> truth == unit(1)
    assert add_1(-1) >> truth is Nothing
    assert truth(False) >> double is Nothing
    assert double([]) >> truth is Nothing


def test_maybe_decorator_with_value():
    @maybe(nothing_on_value=None)
    def truth(x):
        return x

    assert truth(42) is not Nothing
    assert truth('') is not Nothing
    assert truth(0) is not Nothing
    assert truth(False) is not Nothing
    assert truth(None) is Nothing


def test_maybe_decorator_combined():
    @maybe(predicate=bool, nothing_on_value=42)
    def wrap(x):
        return x

    assert wrap(True) == unit(True)
    assert wrap(False) is Nothing
    assert wrap('something') == unit('something')
    assert wrap('') is Nothing
    assert wrap([False]) == unit([False])
    assert wrap([]) is Nothing
    assert wrap(1) == unit(1)
    assert wrap(0) is Nothing
    assert wrap(None) is Nothing
    assert wrap(42) is Nothing


def test_maybe_decorator_none_exception():
    @maybe(nothing_on_exception=None)
    def div(a, b):
        return a / b

    with pytest.raises(ZeroDivisionError):
        div(42, 0)


def test_maybe_decorator_empty_seq_exception():
    for empty in ([], tuple(), set()):
        @maybe(nothing_on_exception=empty)
        def div(a, b):
            return a / b

        with pytest.raises(ZeroDivisionError):
            div(42, 0)


def test_maybe_decorator_specific_exception():
    @maybe(nothing_on_exception=ZeroDivisionError)
    def div(a, b):
        return a / b

    assert div(42, 0) is Nothing


def test_maybe_decorator_specific_exception_tuple():
    @maybe(nothing_on_exception=(IOError, ZeroDivisionError))
    def div(a, b):
        if a < 0:
            raise IOError
        return a / b

    assert div(42, 0) is Nothing
    assert div(-42, 2) is Nothing


def test_first():
    assert first([Nothing, Just(42)]) == Just(42)
    assert first([Just(42), Just(43)]) == Just(42)
    assert first([Nothing, Nothing]) == Nothing
    assert first([]) == Nothing


def test_first_default():
    assert first([Nothing, Nothing], default=Just(42)) == Just(42)


def test_first_predicate():
    assert first([False, 0, 2, 1], predicate=bool) == Just(2)
    assert first([False, 0, ''], predicate=bool) == Nothing
    assert first(range(100), predicate=lambda x: x > 50) == Just(51)
    assert first(range(100), predicate=lambda x: x > 100) == Nothing


def test_first_wrap_just_only_if_not_already():
    assert first([False, True], predicate=bool) == Just(True)
    assert first([False, Just(True)], bool) != Just(Just(True))
    assert first([False, Just(True)], bool) == Just(True)


def test_first_is_lazy():
    def once():
        yield Just(42)
        raise Exception

    assert first(once()) == Just(42)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Try
from smonad.types import List

testee = [
    Identity,
    Maybe,
    Try,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'applicative' in metafunc.funcargnames:
        metafunc.parametrize('applicative', testee, ids=ids)


def test_unit(applicative):
    unit = applicative.unit
    # const
    c = lambda _: 42
    # add_1
    a = 1 .__radd__
    # double
    d = 2 .__rmul__
    f = hash
    # identity
    i = lambda a: a
    for n in test_range:
        # unit . f = fmap f . unit
        assert unit(c(n)) == unit(n).fmap(c)
        assert unit(a(n)) == unit(n).fmap(a)
        assert unit(d(n)) == unit(n).fmap(d)
        assert unit(f(n)) == unit(n).fmap(f)
        assert unit(i(n)) == unit(n).fmap(i)



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
"""smonad.types.monadplus - The MonadPlus Class."""

from . import Monad


class MonadPlus(Monad):
    """The MonadPlus Class.

    Monads that also support choice and failure.
    """
    # Associative Operator
    def __add__(self, monad):
        """The associative operator ``+``"""
        return self.plus(monad)

    # Associative Operation
    def plus(self, monad):
        """The Associative operation."""
        raise NotImplementedError

    #: The identity of ``plus``.
    #:
    #: This property should be a singleton, the following must be ``True``::
    #:
    #:    MP.zero is MP.zero
    #:
    #: It should satisfy the following law, left zero
    #: (notice the bind operator is haskell's ``>>=`` here)::
    #:
    #:    zero >>= f = zero
    zero = NotImplemented



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
from smonad.types import Null


def test_null_repr():
    # for readability of autodoc-generated documents
    assert repr(Null) == 'Null'


def test_null_callable():
    assert Null() is Null



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2014, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.
import pytest

from smonad.mixins import Ord
from smonad.types import Identity
from smonad.types import Maybe
from smonad.types import Failure, Success
from smonad.types import List

testee = [
    Identity,
    Maybe,
    Failure,
    Success,
    List,
]
test_range = range(-100, 100)
ids = [t.__name__ for t in testee]


def pytest_generate_tests(metafunc):
    if 'cls' in metafunc.funcargnames:
        metafunc.parametrize('cls', testee, ids=ids)


def test_ord_is_abstract():
    with pytest.raises(TypeError):
        Ord() < None


def test_compare(cls):
    for n in test_range:
        assert cls(n) == cls(n)
        assert cls(n) != cls(n + 1)


def test_ordering(cls):
    for n in test_range:
        assert (cls(n) < cls(n)) is False
        assert (cls(n) > cls(n)) is False
        m = cls(n)
        assert (m < m) is False
        assert (m > m) is False
        assert cls(n) < cls(n + 1)
        assert cls(n) > cls(n - 1)
